@@ -1,34 +1,52 @@
 makeRules <-
-function(X,Y, nodes=1000, addZ=NULL, nodesize=5, maxinter=2, onlyinter=NULL, silent=FALSE){
+function(X,Y, nodes=1000, addZ=NULL, nodesize=5, maxinter=2, onlyinter=NULL, silent=FALSE, levelvec=NULL){
+
+
+  if(is.null(levelvec)){
+
+    levelvec <- list()
+    for (k in 1:ncol(X)){
+      if(!class(X[,k])%in%c("numeric","factor")){
+        if(!silent) cat("\n", paste("converting ",class(X[,k]), " variable `",colnames(X)[k],"' to numeric vector in current version ..."),sep="")
+        X[,k] <- as.numeric(X[,k])
+      }
+      if(class(X[,k])=="factor"){
+        levelvec[[k]] <- levels(X[,k])
+      }else{
+        levelvec[[k]] <- numeric(0)
+      }
+    }
+  }
   
   n <- nrow(X)
   ZRULES <- if(!is.null(addZ)) addZ[["nodes"]] else list()
   maxloop <- max(100, round(nodes/10)*2 )
   loopc <- 0
   while(length(ZRULES) < nodes & (loopc <- (loopc + 1))<maxloop){
-    rf <- randomForest( X, Y + 0.0001*sd(Y)*rnorm(length(Y)), ntree=10, nodesize=nodesize, keep.forest=TRUE, keep.inbag=FALSE, subsample= min(100, round(nrow(X)/2) ), replace=FALSE)
+    rf <- randomForest( X, Y + 0.01*sd(Y)*rnorm(length(Y)), ntree=30, nodesize=nodesize, keep.forest=TRUE, keep.inbag=FALSE, subsample= max(min(round(n/2),10), round(n/5)) , replace=FALSE)
     
     treelist <- list()
     nx <- 0
     
     for (k in 1:rf$ntree){
       tree <- getTree(rf,k=k)
-      treelarge <- cbind(tree,rep(0,nrow(tree)),rep(0,nrow(tree)))
-      colnames(treelarge) <- c(colnames(tree),"level","position")
-      treelarge <- makepostree(treelarge,1,1,0.5)
+      treelarge <- cbind(tree,rep(0,nrow(tree)))
+      colnames(treelarge) <- c(colnames(tree),"level")
+      treelarge <- makepostree(treelarge,1,1)
       treelist[[k]] <- treelarge
       nx <- nx + sum(treelarge[,"status"]==-3)
     }
     
     for (treec in 1:length(treelist)){
       tree <- treelist[[ treec ]]
-      ZRULES <- c(ZRULES, getRULES(tree,ZRULES,maxinter=maxinter) )
+      ZRULES <- c(ZRULES, getRULES(tree,ZRULES,maxinter=maxinter,levelvec=levelvec) )
     }
-      
+
+    
     non <- sapply(lapply(ZRULES,attr,"n"),is.null)
     if(any(non)){
       for (k in which(non)){
-        ind <- getsamples(ZRULES[[k]],X)
+        ind <- getsamples(ZRULES[[k]],X,levelvec)
         attr(ZRULES[[k]],"n") <- length(ind)
         attr(ZRULES[[k]],"mean") <- mean(Y[ind]) 
       }
@@ -53,6 +71,9 @@ function(X,Y, nodes=1000, addZ=NULL, nodesize=5, maxinter=2, onlyinter=NULL, sil
       }
     }
   }
+  attr(ZRULES,"levelvec") <- levelvec
+
+  
   if(length(ZRULES)<0.95* nodes) warning(paste("Could not generate desired ",nodes,"nodes in reasonable time. Working  with",length(ZRULES)," instead"))
 
   
@@ -65,27 +86,29 @@ function(X,Y, nodes=1000, addZ=NULL, nodesize=5, maxinter=2, onlyinter=NULL, sil
     attr(ZRULES[[lZ+1]],"mean") <- mean(Y)
     attr(ZRULES[[lZ+1]],"depth") <- 0
   }
-  
-  conn <- getconnection(ZRULES,X)
+  attri <- attributes(ZRULES)
   ord <- order(sapply(ZRULES,attr,"n"))
   ZRULES <- ZRULES[ord]
   if(!silent) cat("\n", "total number of nodes in initial set                   :", length(ZRULES))
+  attributes(ZRULES) <- attri
   
-  connold <- conn[ord,ord]
-  conn <- diag(1/diag(connold)) %*% connold
-  diag(conn) <- 0
-  conn <- pmin(conn,t(conn))
-  conn[lower.tri(conn)] <- 0
-  maxcon <- apply(conn,2,max)
-  rem <- which(maxcon>0.999 & sapply(ZRULES,attr,"depth")>0)
-  if(length(rem)>0){
+  I <- getI(ZRULES,X,mode="mean")$I
+  hash <- round(100000* as.numeric(t(I)%*%rnorm(n)))
+  tabb <- table(hash)
+  if(any(tabb>1)){
+    uniquehash <- as.numeric(names(tabb))
+    rem <- numeric(0)
+    doub <- which(tabb>1)
+    for (khc in 1:length(doub)){
+      ind <- which(hash==uniquehash[doub[khc]])
+      rem <- c(rem,ind[1:(length(ind)-1)])
+    }
+    attri <- attributes(ZRULES)
     ZRULES <- ZRULES[-rem]
-    ord <- ord[ - rem]
-    conn <- connold[-rem,-rem, drop=FALSE]
+    attributes(ZRULES) <- attri
   }
   
   if(!silent) cat("\n", "total number of nodes after removal of identical nodes :", length(ZRULES),"\n")
-  attr(ZRULES,"connection") <- conn
   
   return(ZRULES)
 }
